#!/bin/bash

# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available at
# <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
# license, available in the LICENSE.TXT file accompanying this
# software.  These details are also available at
# <http://openib.org/license.html>.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS                               
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
#
# Description: IBGD RPMs building script
#

usage()
{
cat << EOF

    `basename $0` [ -c <config_file> ]

EOF
}

cd ${0%*/*}
CWD=`pwd`

INTERACTIVE=1

if [ ! -e ./build_env.sh ]; then
    echo "ERROR: $CWD/build_env.sh file is missing"
    exit 1
fi

while [ $# -gt 0 ]
do
    case $1 in
            -c)
                shift
                CONFIG=$1
                INTERACTIVE=0
            ;;
            *)
                usage
                exit 1
            ;;
    esac
    shift
done

. $CWD/build_env.sh

# Environment variables definition
BUILD_COUNTER=0
EXISTING_RPMS=0


# HCA driver installation
openib()
{
IFS=' '
#if [ ! -f ${RPMS}/${OPENIB_NAME}-${OPENIB_VER}-${RPMK_VER}.${build_arch}.rpm ]; then
    if ! check_build_req openib err; then
        exit 2
    fi    
    cd $BUILD_TMP
    ex_silent "$RM -f ${OPENIB_SOURCE##*/}"
    echo
    echo "Building ${OPENIB_NAME} RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${OPENIB_SOURCE}"
    local OPENIB_SRCDIR=$BUILD_TMP/`echo ${OPENIB_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    if [ ! -z "${VAPI_VER}" ]; then
        warn_echo "Forcing to use vapi-${VAPI_VER}"
        vapi_str="--vapi ${VAPI_VER}"
    else
        vapi_str=""
    fi        
    ex "${OPENIB_SRCDIR}/build_rpm.sh --prefix ${STACK_PREFIX} --build_root ${BUILD_ROOT} ${vapi_str}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${OPENIB_NAME}-${OPENIB_VER}-${RPMK_VER}.${build_arch}.rpm $RPMS"
    ex_silent "$RM -rf ${OPENIB_SRCDIR}"
    cd -
    let BUILD_COUNTER++
#else
#    echo \"${OPENIB_NAME}-${OPENIB_VER}-${RPMK_VER}.${build_arch}.rpm\" already exists
#    let EXISTING_RPMS++
#fi

## Set permissions
#$CHMOD -R +rw ${OPENIB_DEST}

return 0
}


# OpenSM installation
opensm()
{
IFS=' '
#if [ ! -f ${RPMS}/${OPENSM_NAME}-${OPENSM_VER}-${RPMK_VER}.${build_arch}.rpm ]; then
    if ! check_build_req opensm err; then
        exit 2
    fi    
    cd $BUILD_TMP
    ex_silent "$RM -f ${OPENSM_SOURCE##*/}"
    echo
    echo "Building ${OPENSM_NAME} RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${OPENSM_SOURCE}"
    local OPENSM_SRCDIR=$BUILD_TMP/`echo ${OPENSM_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    if [ -d "/lib/modules/${K_VER}/source/" ]; then
        MTHOME=/lib/modules/${K_VER}/source/drivers/infiniband/hw/mellanox-hca
        TSHOME=/lib/modules/${K_VER}/source/drivers/infiniband/include
    else
        MTHOME=/lib/modules/${K_VER}/build/drivers/infiniband/hw/mellanox-hca
        TSHOME=/lib/modules/${K_VER}/build/drivers/infiniband/include
    fi
    ex "${OPENSM_SRCDIR}/build_rpm.sh -b ${BUILD_ROOT} -d ${OPENSM_DEST} -m ${BUILD_ROOT}${OPENIB_DEST} -t ${BUILD_ROOT}${TSHOME} -v ${VENDOR}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${OPENSM_NAME}-${OPENSM_VER}-${RPMK_VER}.${build_arch}.rpm $RPMS"
    ex_silent "$RM -rf ${OPENSM_SRCDIR}"
    cd -    
    let BUILD_COUNTER++
#else
#    echo \"${OPENSM_NAME}-${OPENSM_VER}-${RPMK_VER}.${build_arch}.rpm\" already exists
#    let EXISTING_RPMS++    
#fi

return 0
}

# HCA Tools installation
mst()
{
IFS=' '
    if ( $RPM -q ${MST_NAME}-${ARCH} > $NULL 2>&1 ); then
        ex "$RPM -e ${MST_NAME}-${ARCH}"
    fi
#if [ ! -f ${RPMS}/${MST_NAME}-${ARCH}-${MST_VER}-${RPMK_VER}.${build_arch}.rpm ]; then
    if ! check_build_req mst err; then
        return 2
    fi    
    cd $BUILD_TMP
    ex_silent "$RM -f ${MST_SOURCE##*/}"
    echo
    echo "Building ${MST_NAME}-${ARCH} RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${MST_SOURCE}"
    local MST_SRCDIR=$BUILD_TMP/`echo ${MST_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    ex "${MST_SRCDIR}/build_rpm.sh"
    ex "$RPM -ihv ${RPM_DIR}/RPMS/$build_arch/${MST_NAME}-${ARCH}-${MST_VER}-${RPMK_VER}.${build_arch}.rpm"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${MST_NAME}-${ARCH}-${MST_VER}-${RPMK_VER}.${build_arch}.rpm $RPMS"
    ex_silent "$RM -rf ${MST_SRCDIR}"
    cd -
    let BUILD_COUNTER++
#else
#    echo \"${MST_NAME}-${ARCH}-${MST_VER}-${RPMK_VER}.${build_arch}.rpm\" already exists
#    let EXISTING_RPMS++    
#fi

return 0
}

# IBADM installation
ibadm()
{
IFS=' '
#if [ ! -f ${RPMS}/${IBADM_NAME}-${IBADM_VER}-${RPMK_VER}.${build_arch}.rpm ]; then
    if ! check_build_req ibadm err; then
        exit 2
    fi    
    cd $BUILD_TMP
    ex_silent "$RM -f ${IBADM_SOURCE##*/}"
    echo
    echo "Building ${IBADM_NAME} RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${IBADM_SOURCE}"
    local IBADM_SRCDIR=$BUILD_TMP/`echo ${IBADM_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    ex "${IBADM_SRCDIR}/build_rpm.sh  -b ${BUILD_ROOT} -d ${IBADM_DEST} -stack ibgd --prefix ${STACK_PREFIX}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${IBADM_NAME}-${IBADM_VER}-${RPMK_VER}.${build_arch}.rpm $RPMS"
    ex_silent "$RM -rf ${IBADM_SRCDIR}"
    cd -    
    let BUILD_COUNTER++
#else
#    echo \"${IBADM_NAME}-${IBADM_VER}-${RPMK_VER}.${build_arch}.rpm\" already exists
#    let EXISTING_RPMS++    
#fi

return 0
}

             
# NCSA MPI installation
mpi_ncsa()
{
IFS=' '
local VMI_RPM=${MPI_NCSA_VMI_NAME}-${MPI_NCSA_VMI_VER}-${RPMK_VER}.${build_arch}.rpm
local NCSA_RPM=${MPI_NCSA_MPICH_NAME}-${MPI_NCSA_MPICH_VER}-${RPMK_VER}.${build_arch}.rpm

#if [[ ! -f ${RPMS}/${VMI_RPM} || ! -f ${RPMS}/${NCSA_RPM} ]]; then
    if ! check_build_req mpi_ncsa err; then
        return 2
    fi    
    
	# Add libcurl.so link (W/A for Rocks 0.3.3)
	if [ -d /usr/lib ]; then
	        if [ ! -h /usr/lib/libcurl.so ]; then
	                cd /usr/lib
	                ln -snf libcurl.so.2.0.2 libcurl.so
	                cd -
	        fi
	fi

	if [ -d /usr/lib64 ]; then
	        if [ ! -h /usr/lib64/libcurl.so ]; then
	                cd /usr/lib64
	                ln -snf libcurl.so.2.0.2 libcurl.so
	                cd -
	        fi
	fi


    cd $BUILD_TMP
    ex_silent "$RM -f ${NCSA_SOURCE##*/}"
    echo
    echo "Building NCSA RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${NCSA_SOURCE}"
    local NCSA_SRCDIR=$BUILD_TMP/`echo ${NCSA_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    ex "${NCSA_SRCDIR}/build_vmi_rpm.sh -b ${BUILD_ROOT} --prefix ${STACK_PREFIX}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${VMI_RPM} $RPMS"
    ex "env PGF_77=$PGF_77 PGF_90=$PGF_90 ${NCSA_SRCDIR}/build_mpich_rpm.sh -b ${BUILD_ROOT} --prefix ${STACK_PREFIX}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${NCSA_RPM} $RPMS"
    ex_silent "$RM -rf ${NCSA_SRCDIR}"
    cd -
    let BUILD_COUNTER++
#else
#    echo \"${VMI_RPM}\" and \"${NCSA_RPM}\" already exists
#    let EXISTING_RPMS++    
#fi

return 0
}

# OSU MPI installation
mpi_osu()
{
IFS=' '
local OSU_RPM=${MPI_OSU_MPICH_NAME}-${MPI_OSU_MPICH_VER}-${RPMK_VER}.${build_arch}.rpm

#if [ ! -f ${RPMS}/${OSU_RPM} ]; then
    if ! check_build_req mpi_osu err; then
        return 2
    fi    
    cd $BUILD_TMP
    ex_silent "$RM -f ${OSU_SOURCE##*/}"
    echo
    echo "Building OSU RPM. Please wait..."
    echo
    ex_silent "tar xzvf ${OSU_SOURCE}"
    local OSU_SRCDIR=$BUILD_TMP/`echo ${OSU_SOURCE##*/} | sed -e 's/.tgz//' | tr -d [:space:]`
    ex "env PGF_77=$PGF_77 PGF_90=$PGF_90 ${OSU_SRCDIR}/build_rpm.sh -b ${BUILD_ROOT} --prefix ${STACK_PREFIX}"
    ex "$MV -f ${RPM_DIR}/RPMS/$build_arch/${OSU_RPM} $RPMS"
    ex_silent "$RM -rf ${OSU_SRCDIR}"
    cd -
    let BUILD_COUNTER++
#else
#    echo \"${OSU_RPM}\" already exists
#    let EXISTING_RPMS++    
#fi

return 0
}

make_build()
{
IFS=' '
# # Install packages required for IBGD operation
# # Like ( swig1.1p5, tcl8.3.2 )
# install_packages

echo   
echo "Building InfiniBand Software RPMs. Please wait..."
echo

IFS=';'
for package in $REQUIRED_PACKAGES $OPTIONAL_PACKAGES
do
      pack=`echo $package | cut -d ':' -f 1 | tr -d [:space:]`
      if [ "`eval echo \$\{${pack}\}`" == "y" ]; then   
            $pack
      fi
done
IFS=' '

return 0
}

# Main

main()
{
IFS=' '
check_user

# Set network environment configuration                  
if [ -f /etc/SuSE-release ]; then
    DISTRIBUTION="SuSE"
    
    # Check termcap library link
    if [[ ! -e /usr/lib/libtermcap.so && -e /usr/lib/libtermcap.so.2.0.8 ]]; then
        ex_silent "ln -s /usr/lib/libtermcap.so.2.0.8 /usr/lib/libtermcap.so"
    fi
    
elif [ -f /etc/redhat-release ]; then
    DISTRIBUTION="redhat"
elif [ -f /etc/rocks-release ]; then
    DISTRIBUTION="Rocks"
else
    DISTRIBUTION="Unknown"
fi

if [ $INTERACTIVE -eq 1 ]; then
   show_menu "build_menu"
	
        read ans_inst
        case $ans_inst in
                2)   
                     echo
                     echo "Checking dependencies. Please wait ..."
                     echo

                     # Check for Infiniband software of other vendors
                     local vendors="topspin iview_fm"
                     local vrpms=""
                     
                     for vendor in $vendors
                     do
                        vrpms=`$RPM -qa | grep $vendor`
                        if [ -n "$vrpms" ]; then
                            echo
                            err_echo "You have to remove ${vendor} Infiniband software before continuing:"
                            echo
                            echo "$vrpms"
                            echo
                            exit 9
                        fi
                     done
                    
                     # Check dependencies and print Warning messages
                     # Check if kernel sources exists (needed for packages compilation)
                     if ! test -d /lib/modules/${K_VER}/build; then
                         echo
                         warn_echo "There is no sources for ${K_VER} kernel installed.
                                    Please install ${K_VER} kernel sources to build RPMs on this system"
                         echo
                         getch
                     fi
                     
                     local req_status=0
       
                     IFS=';'
                     for package in $REQUIRED_PACKAGES
                     do
                         pack=`echo $package | cut -d ':' -f 1 | tr -d [:space:]`
                         check_build_req $pack err
                         let req_status=$req_status+$?
                     done
                     IFS=' '
                     
                     IFS=';'
                     for package in $OPTIONAL_PACKAGES
                     do
                         pack=`echo $package | cut -d ':' -f 1 | tr -d [:space:]`
                         check_build_req $pack warn
                     done
                     IFS=' '
                     
                     # Exit if there is missing required software for mandatory packages
                     if [ $req_status -gt 0 ]; then
                        getch
                        exit 7
                     fi   
                     
                     # Print IBGD Package content
                     echo 
                     echo " IBGD Software Packages:"
                     echo
                     echo " The following packages will be built:"
                     echo
                     IFS=';'
                     for package in $REQUIRED_PACKAGES
                     do
                             p_name=`echo $package | cut -d : -f 2`
                             echo "     $p_name"
                     done
                     IFS=' '
                     echo
                     echo " The following packages are optional:"
                     echo
                     IFS=';'
                     for package in $OPTIONAL_PACKAGES
                     do
                             p_name=`echo $package | cut -d : -f 2`
                             echo "     $p_name"
                     done
                     IFS=' '


                     echo
                     echo "Going to build IBGD RPMs"
                     echo "The RPM build process uses a temporary directory"
                     echo
                     read -p "Please enter the temporary directory [${BUILD_ROOT}]:"
                     BUILD_ROOT=${REPLY:-$BUILD_ROOT}

                     if [ "${BUILD_ROOT}" != "${BUILD_ROOT_DFT}" ]; then
                            BUILD_ROOT=${BUILD_ROOT}/IBGD
                     fi

                     [ -d $BUILD_ROOT ] && $RM -rf $BUILD_ROOT
                     
                     check_space_req "$BUILD_SPACE_REQ" "$BUILD_ROOT"
                     
                     echo
                     echo "The build process requires the target installation directory."
                     echo
                     echo "   Note: The build process does not create nor touch the target installation directory."
                     echo "         The directory will only be created when you actually install the IBGD RPMs."
                     echo
                     read -p "Please enter the IBGD installation directory [${STACK_PREFIX}]:"
                     STACK_PREFIX=${REPLY:-$STACK_PREFIX}
                     echo   
                     # Set  parameters with new values
                     . $CWD/build_env.sh
                     echo
                     pass_echo "Chosen temporary directory for IBGD RPMs is: $BUILD_ROOT"
                     pass_echo "RPM packages will be stored under: $PKG_SOURCE_DIR/RPMS"
                     pass_echo "IBGD Software is configured to be installed under: ${STACK_PREFIX}"
                     echo
                     read -p "Do you want to continue? [Y/n]:" ans_r
                     if ! [[ "$ans_r" == "" || "$ans_r" == "y" || "$ans_r" == "Y" || "$ans_r" == "yes" ]]; then
                         return 1
                     fi

                     
                     if [ ! -d "$BUILD_TMP" ]; then
                        ex_silent "$MKDIR -p $BUILD_TMP"
                     fi
   
                     make_config install
                     if [[ "$mpi_ncsa" == "y" || "$mpi_osu" == "y" ]]; then
                         # Set fortran environment for NCSA and OSU MPI                                    
                         if ! set_fortran_env; then
                            warn_echo "Failed to locate Fortran compiler. NCSA and OSU MPI will not be installed"
                            export mpi_ncsa=n
                            export mpi_osu=n
                         fi   
                     fi

                     echo
                     pass_echo "It may take up to 40 minutes to complete the installation process"
                     echo   
                     

                     
                     if make_build; then
                         echo
                         if [ $BUILD_COUNTER -gt 0 ]; then
                            pass_echo "$BUILD_COUNTER RPMs were built"
                         fi
                         echo
                         pass_echo "Build process finished ..."

                         echo
                         read -p  "Do you want to remove the temporary build directory $BUILD_ROOT [Y/n]?" ans_r
                         if [[ "$ans_r" == "" || "$ans_r" == "y" || "$ans_r" == "Y" || "$ans_r" == "yes" ]]; then
                             ex_silent "$RM -rf $BUILD_ROOT"
                         fi
                         
                         getch
                     fi
                ;;
                1)
                    view_readme
                ;;    
                Q|q)
                    close_log    
                    exit 0
                ;;
                *)
                    return 1
                ;;
        esac
else

    # Check dependencies and print Warning messages
    # Check if kernel sources exists (needed for packages compilation)
    if ! test -d /lib/modules/${K_VER}/build; then
        echo
        warn_echo "There is no sources for ${K_VER} kernel installed.
                   Please install ${K_VER} kernel sources to build RPMs on this system"
        echo
        exit 1
    fi

    # Check $CONFIG file
    if [ "${openib}${opensm}${mst}${ibadm}" != "yyyy" ]; then
        
        if [ "${openib}${opensm}${mst}${ibadm}" != "nnnn" ]; then
            echo
            echo "  openib (Infiniband HCA Driver), opensm (Infiniband Subnet Manager), mst (Mellanox Tools)"
            echo "  and ibadm (Infiniband Administration Package) are mandatory for Infiniband operation"
            echo "  Please set \"openib=y\" , \"opensm=y\" , \"mst=y\" and \"ibadm=y\" in the $CONFIG file to Install Driver,"
            echo "  or set all packages to \"n\" to Uninstall all Infiniband Software"
            echo
            exit 5
        else    
            if [ "${mpi_ncsa}${mpi_osu}" != "nn" ]; then
                echo
                echo "  To Add packages set \"openib=y\" , \"opensm=y\" , \"mst=y\" and \"ibadm=y\" in the $CONFIG file"
                echo
                echo "  To remove packages set all variables to \"n\" in the $CONFIG file"
                echo
                exit 6
            fi
        fi
        
    fi

    if [ "${openib}${opensm}${mst}${ibadm}${mpi_ncsa}${mpi_osu}" == "nnnnnn" ]; then
        echo "Nothing to build"
        exit 1
    else    
        if [[ "$mpi_ncsa" == "y" || "$mpi_osu" == "y" ]]; then
            # Set fortran environment for MPI                                    
            if ! set_fortran_env; then
                 warn_echo "Failed to locate Fortran compiler. NCSA and OSU MPI will not be installed"
                 export mpi_ncsa=n
		 export mpi_osu=n
                 perl -ni -e "s@(mpi_ncsa=).*@\$1${mpi_ncsa}@; print" $CONFIG
                 perl -ni -e "s@(mpi_osu=).*@\$1${mpi_osu}@; print" $CONFIG
            fi   
        fi

        local req_status=0

        IFS=';'
        for package in $REQUIRED_PACKAGES
        do
            pack=`echo $package | cut -d ':' -f 1 | tr -d [:space:]`
            check_build_req $pack err
            let req_status=$req_status+$?
        done
        IFS=' '
        
        IFS=';'
        for package in $OPTIONAL_PACKAGES
        do
            pack=`echo $package | cut -d ':' -f 1 | tr -d [:space:]`
            check_build_req $pack warn
        done
        IFS=' '
        
        # Exit if there is missing required software for mandatory packages
        if [ $req_status -gt 0 ]; then
           exit 7
        fi   

        if [ ! -d  "$BUILD_ROOT" ]; then
            ex_silent "$MKDIR -p $BUILD_ROOT"
        fi

        check_space_req "$BUILD_SPACE_REQ" "$BUILD_ROOT"
        
        if [ ! -d "$BUILD_TMP" ]; then
           ex_silent "$MKDIR -p $BUILD_TMP"
        fi
   
        if make_build; then
             echo
             if [ $BUILD_COUNTER -gt 0 ]; then
                pass_echo "$BUILD_COUNTER RPMs were built"
             fi
             if [ $EXISTING_RPMS -gt 0 ]; then
                pass_echo "$EXISTING_RPMS RPMs already exists"
             fi
             echo
             
             pass_echo "Building process finished ..."
        fi
    fi        

    # Remove BUILD_TMP directory
    if [ "$BUILD_ROOT" == "${BUILD_ROOT_DFT}" ]; then
       ex_silent "$RM -rf $BUILD_ROOT"
    fi   
#     if [ $NEED_REBOOT -ne 0 ]; then
#         echo
#         echo    "Please reboot the system"
#         echo
#     fi
fi

return 0
}

open_log
while true
do
    main
    if [ $INTERACTIVE -eq 0 ]; then
        exit 0
    fi    
done    
