#!/bin/bash 

# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available at
# <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
# license, available in the LICENSE.TXT file accompanying this
# software.  These details are also available at
# <http://openib.org/license.html>.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
#
# Description: RPM build script
#

build_root=/var/tmp/IBGD
STACK_PREFIX=/usr/local/ibgd
IBADMHOME=${STACK_PREFIX}/apps/ibadm
STACK=ibgd

usage()
{
cat << EOF

    `basename $0` [ -b <BUILD_ROOT> ] [ -d <install-dir> ] [-stack <stack type>] [--prefix <Stack_Prefix>]
			
		The default install-dir is $IBADMHOME
		The default BUILD_ROOT  is $build_root
                -stack option : openib ; hpc=IBHPC ; mlx=mvapi (default $STACK)
                --prefix option : stack installation prefix (default $STACK_PREFIX)

EOF
}

while [ $# -gt 0 ]
do
    case $1 in
            -d)
                shift
                IBADMHOME=$1
            ;;
	    -b)
		shift
		build_root=$1
	    ;;
            -stack)
                shift
                STACK=$1
            ;;
            --prefix)
                shift
                STACK_PREFIX=$1
            ;;        
            *)
                usage
                exit 1
            ;;
    esac            
    shift
done

BUILD_ENV="env build_root=$build_root IBADMHOME=$IBADMHOME STACK=$STACK STACK_PREFIX=$STACK_PREFIX"

cd ${0%*/*}

TMPFILE=/tmp/$$.tmp
SOURCE_DIR=`pwd`
KERNEL=`uname -r | sed -e "s/\-/\_/g"`
OSM_DIR=${PWD##*/}
NAME=`echo ${PWD##*/} | cut -d \- -f 1`
VERSION=`echo ${PWD##*/} | cut -d \- -f 2- | tr -s '-' '_'`
RELVER=`echo ${PWD##*/} | cut -d \- -f 2-`
BUILD_REQUIRES="glib glib-devel glibc glibc-devel"

# Check kernel sources existence
if ! test -d /lib/modules/`uname -r`/build; then
	echo In order to build RPM kernel sources need to be installed
	exit 1
fi

# Check dependencies
for pack in $BUILD_REQUIRES
do            
    if ! ( rpm -q $pack > /dev/null 2>&1 ); then
	echo In order to build RPM $pack need to be installed
	exit 1
    fi
done


build_arch=`rpm --showrc | grep -w "build arch" | cut -d : -f 2 | tr -d " "`
spec="ibadm.spec"

if [ -d /usr/src/redhat ]; then
	RPM_DIR=/usr/src/redhat
elif [ -d /usr/src/packages ]; then
	RPM_DIR=/usr/src/packages
else
	echo Build rpm on this system is not supported...
	exit 1
fi

if [ -f $RPM_DIR/SPECS/$spec ]; then
	rm -f $RPM_DIR/SPECS/$spec 
fi

# Update Package name, version and release in the spec file
sed -e "s/XXXXXX/$VERSION/g" -e "s/YYYYYY/$NAME/g" -e "s/ZZZZZZ/$RELVER/g" $spec > $RPM_DIR/SPECS/$spec

cd ..
tar czvf ${NAME}-${RELVER}.tgz $OSM_DIR > /dev/null 2>&1
mv -f ${NAME}-${RELVER}.tgz $RPM_DIR/SOURCES/

if [ -f /etc/SuSE-release ]; then
   if [ `grep VERSION /etc/SuSE-release | cut -d = -f 2 | cut -d . -f 1 | tr -d [:space:]` -ge 9 ]; then
	$BUILD_ENV rpmbuild -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
   else	
	$BUILD_ENV rpm -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
   fi
elif [[ -f /etc/redhat-release || -f /etc/rocks-release ]]; then
	$BUILD_ENV rpmbuild -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
else
        echo Build rpm on this system is not supported...
        exit 1
fi

if [ $? != 0 ]; then
    echo RPM building failed...
    exit 1
fi 
 
rm -f $RPM_DIR/SOURCES/$OSM_DIR.tgz
rm -f $RPM_DIR/SPECS/$spec

if [ ! -f $RPM_DIR/RPMS/$build_arch/${NAME}-${VERSION}-${KERNEL}.${build_arch}.rpm ] ; then
	echo RPM building failed...
	exit 1
else
	echo RPM: $RPM_DIR/RPMS/$build_arch/${NAME}-${VERSION}-${KERNEL}.${build_arch}.rpm 
fi

exit 0

