#!/bin/bash 
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available at
# <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
# license, available in the LICENSE.TXT file accompanying this
# software.  These details are also available at
# <http://openib.org/license.html>.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
#

# pointing to the dir where this install script resides
PACKAGE_ORIG_DIR=`cd ${0%*/*};pwd`

########################################################################
#
# FUNCTIONS:
#

#Report: if uninstall failed
function uinstall_failed() {
	echo "Uninstall failed ! (see $UNINSTLOG for details)"
	exit 1
}

function cleanup_ibadm_files() {
    # Clean old distribution
    binApps="ibadm ibis ibns ibbs ibburn ibfwmgr ibtopogen ibmon ibls obbs-pci 
            unify_image_section t2a mic aimage ibdmtr ibdmchk ibdmsim 
            ibdmsh ibcon ibnlparse"

    echo " Removing Executables from : $CONFIG_ROOT/bin." 
    for f in $binApps; do 
        rm -f ${CONFIG_ROOT}/bin/$f 2&>1 > /dev/null; 
        if [ $? == 0 ]; then
            echo "Removed : ${CONFIG_ROOT}/bin/$f" >> $UNINSTLOG
        fi
    done
   
    echo " Removing Includes Files from : $CONFIG_ROOT/include."
    rm -rf ${CONFIG_ROOT}/include/ibdm 2&>1 > /dev/null
    if [ $? == 0 ]; then
        echo "Removed : ${CONFIG_ROOT}/include/ibadm" >> $UNINSTLOG
    fi
    
    echo " Removing Old IABDM Distribution Libs from : $CONFIG_ROOT/lib."
    libs="ibbbm1.0 ibconf1.0 ibcr1.0 ibdm1.0 ibis1.0 ibpm1.0 ibsac1.0 
          g_file1.0 g_puts1.0 ibadmConn1.0 inifile1.0 
          libibdm.a libibdm.la libibdm.so libibdm.so.1 
          libibdm.so.1.1.1 libibdmcom.a libibdmcom.la 
          libibdmcom.so libibdmcom.so.1 libibdmcom.so.1.1.1"
    for f in $libs; do 
        rm -rf ${CONFIG_ROOT}/lib/$f 2&>1 > /dev/null; 
        if [ $? == 0 ]; then
            echo "Removed : ${CONFIG_ROOT}/lib/$f" >> $UNINSTLOG
        fi
    done
    echo " Removing IBADM Libs from : $CONFIG_ROOT/lib/ibadm."
    rm -rf ${CONFIG_ROOT}/lib/ibadm 2&>1 > /dev/null
    if [ $? == 0 ]; then
        echo "Removed : ${CONFIG_ROOT}/lib/ibadm" >> $UNINSTLOG
    fi
    echo " Removing IBADM Man Pages from : /usr/share/man/man1."
    mans="ibadm ibcon ibmon ibls ibadm.conf ibfwmgr ibtopogen"
    for f in $mans; do 
        rm -rf /usr/share/man/man1/$f.1.* 2&>1 > /dev/null; 
        if [ $? == 0 ]; then
            echo "Removed : " /usr/share/man/man1/$f.1.* >> $UNINSTLOG
        fi
    done
}

########################################################################

PLATF=`uname -m`

case $PLATF in
    "i686")
         MY_EXT="x86"
         EXPECTED_MAKELOG_SIZE=37574
         EXPECTED_MAKEINSTALLLOG_SIZE=37574
         ;;
    "ia64")
         MY_EXT="ia64"
         EXPECTED_MAKELOG_SIZE=37574
         EXPECTED_MAKEINSTALLLOG_SIZE=37574
         ;;
    "x86_64")
         MY_EXT="x86_64"
         EXPECTED_MAKELOG_SIZE=37574
         EXPECTED_MAKEINSTALLLOG_SIZE=37574
         ;;
    *)
         echo "Unsupported platform. IBADM Currently supports only x86, ia64 platforms"
         exit 1
esac

# Initialize default before parsing parameters
CONFIG_ROOT=/usr
INSTALL_ROOT=/usr/mellanox/ibadm
INSTALL_LIB=/usr/mellanox
NO_CFG=0
DIST=0

# parse parameters
while [ "$1" ]; do
#  echo "Current parsed param is : $1"
  case $1 in
    "--prefix") 
          CONFIG_ROOT=$2
          shift
          ;;
    "-d") 
          INSTALL_ROOT=$2
          shift
          ;;
    "-nc")
          NO_CFG=1
          echo "<<<<<<<< NO CONFIG CREATION >>>>>>>>>>"
          ;;
    "--dist")
          DIST=1
          echo "<<<<<<<< DISTRIBUTION MODE >>>>>>>>>>"
          ;;

    *)
	  echo "Usage: $0 [--dist][--prefix <install-dir>][-d <bld-dir>][-nc][-b <stack type>]"
     echo "Options:"
     echo "--dist : cause the un-install to remove a distribution tar file."
	  echo "--prefix <dir> : remove bin/lib/include from <dir>"
	  echo "-d <dir> : remove the build tree of IBADM from <dir>" 
	  echo "-nc option : do not remove /etc/ibadm.conf"
      
     exit 1
  esac
  shift
done

export IBADMHOME=$INSTALL_ROOT
# take the src BUILD_ID
BLD_ID="<unknown>"
if test -e ${IBADMHOME}/BUILD_ID ; then
  BLD_ID=`cat ${IBADMHOME}/BUILD_ID`
fi

KERNEL_VER=`uname -r`

UNINSTLOG=/tmp/IBADM.uninstall.log.$$
rm -f $UNINSTLOG
echo "IBADM-$MY_EXT \($BLD_ID\) uninstalled on `date`." >| $UNINSTLOG
echo "Modules uninstall log file:" >> $UNINSTLOG


echo IBADM un-installation script - IBADM Build: $BLD_ID
echo Copyright \(C\) June 2002, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
echo Use of software subject to the terms and conditions detailed in the 
echo file \"LICENSE.txt\".
echo " "


if [ $UID != 0 ]; then
  echo You must be root to un-install IBADM
  exit 1
fi
umask 022   # assure readibility to all users

# Check for previous installations
if [ -n $IBADMHOME ] ; then
  if [ -d $IBADMHOME ] ; then
    echo "Found traces of Mellanox IBADM software installation at $IBADMHOME."
    echo "Current installed IBADM Build ID is $BLD_ID"
    echo "Current installed IBADM Build ID is $BLD_ID" >> $UNINSTLOG
    echo "" >> $UNINSTLOG
    echo -n "Remove currently installed components (y/n) [n] ? "
    read force 
    if [ "$force" != "y" ] && [ "$force" != "yes" ] && [ "$force" != "Y" ] && [ "$force" != "YES" ] ; then
        exit 1
    else
        cleanup_ibadm_files
    fi
  else
    echo "No traces of another Mellanox software installation at $IBADMHOME."
    exit 1
  fi
fi


# cleanup
rm -rf $IBADMHOME

echo "  "
echo -------------------------------------
echo "    IBADM un-installation done."
echo -------------------------------------
echo "  " >> $UNINSTLOG
echo " ---------------------------------- " >> $UNINSTLOG
echo "    IBADM un-installation done." >> $UNINSTLOG
echo " ---------------------------------- " >> $UNINSTLOG
echo "  "
exit 0
